# -*- coding: utf-8 -*-
#
# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)
"""
Profiler Plugin.
"""

# Local imports
from spyder.plugins.profiler.plugin import ProfilerActions
from spyder.plugins.profiler.widgets.main_widget import (
    ProfilerWidgetActions, ProfilerWidgetInformationToolbarSections,
    ProfilerWidgetMainToolbarSections, ProfilerWidgetToolbars)
